'use strict'

//chooses an indefinite aricle 'a/an' for a word
const irregulars = {
  hour: 'an',
  heir: 'an',
  heirloom: 'an',
  honest: 'an',
  honour: 'an',
  honor: 'an',
  uber: 'an', //german u
}
//pronounced letters of acronyms that get a 'an'
const an_acronyms = {
  a: true,
  e: true,
  f: true,
  h: true,
  i: true,
  l: true,
  m: true,
  n: true,
  o: true,
  r: true,
  s: true,
  x: true,
}
//'a' regexes
const a_regexs = [
  /^onc?e/i, //'wu' sound of 'o'
  /^u[bcfhjkqrstn][aeiou]/i, // 'yu' sound for hard 'u'
  /^eul/i,
]

const makeArticle = function(doc) {
  //no 'the john smith', but 'a london hotel'
  if (doc.has('#Person') || doc.has('#Place')) {
    return ''
  }
  //no a/an if it's plural
  if (doc.has('#Plural')) {
    return 'the'
  }
  let str = doc.text('normal').trim()
  //explicit irregular forms
  if (irregulars.hasOwnProperty(str)) {
    return irregulars[str]
  }
  //spelled-out acronyms
  let firstLetter = str.substr(0, 1)
  if (doc.has('^@isAcronym') && an_acronyms.hasOwnProperty(firstLetter)) {
    return 'an'
  }
  //'a' regexes
  for (let i = 0; i < a_regexs.length; i++) {
    if (a_regexs[i].test(str)) {
      return 'a'
    }
  }
  //basic vowel-startings
  if (/^[aeiou]/i.test(str)) {
    return 'an'
  }
  return 'a'
}

module.exports = makeArticle
